
try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

def spl_color():
  #cmd.do('color grey50') # PRP8
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_B5zwo")
        cmd.do("color grey60, BRR2_B5zwo")
        cmd.do("color palegreen, LEA1_B5zwo")
        cmd.do("color palegreen, Msl1_B5zwo")
        cmd.do("color slate, SNU114_B5zwo")
        cmd.do("color forest, U2_B5zwo")
        cmd.do("color density, U5_B5zwo")
        cmd.do("color firebrick, U6_B5zwo")
        cmd.do("color brown, U4_B5zwo")
        cmd.do("color black, Intron_B5zwo")
        cmd.do("color grey50, PRP4_B5zwo")
        cmd.do("color grey50, PRP31_B5zwo")
        cmd.do("color grey50, PRP6_B5zwo")
        cmd.do("color grey50, PRP3_B5zwo")
        cmd.do("color grey50, DIB1_B5zwo")
        cmd.do("color grey50, SNU13_B5zwo")
        cmd.do("color grey50, LSM8_B5zwo")
        cmd.do("color grey50, LSM2_B5zwo")
        cmd.do("color grey50, LSM3_B5zwo")
        cmd.do("color grey50, LSM6_B5zwo")
        cmd.do("color grey50, LSM5_B5zwo")
        cmd.do("color grey50, LSM7_B5zwo")
        cmd.do("color grey50, LSM4_B5zwo")
        cmd.do("color grey50, SNU66_B5zwo")
        cmd.do("color grey60, BUD13_B5zwo")
        cmd.do("color palegreen, Cus1_B5zwo")
        cmd.do("color smudge, HSH155_B5zwo")
        cmd.do("color sand, HSH49_B5zwo")
        cmd.do("color grey60, PML1_B5zwo")
        cmd.do("color palegreen, PRP11_B5zwo")
        cmd.do("color palegreen, RDS3_B5zwo")
        cmd.do("color smudge, RSE1_B5zwo")
        cmd.do("color grey60, SNU17_B5zwo")
        cmd.do("color palegreen, Ysf3_B5zwo")
        cmd.do("color grey50, SMB1_1_B5zwo")
        cmd.do("color grey50, SMB1_2_B5zwo")
        cmd.do("color grey50, SMB1_3_B5zwo")
        cmd.do("color grey50, SME1_1_B5zwo")
        cmd.do("color grey50, SME1_2_B5zwo")
        cmd.do("color grey50, SME1_3_B5zwo")
        cmd.do("color grey50, SMX3_1_B5zwo")
        cmd.do("color grey50, SMX3_2_B5zwo")
        cmd.do("color grey50, SMX3_3_B5zwo")
        cmd.do("color grey50, SMX2_1_B5zwo")
        cmd.do("color grey50, SMX2_2_B5zwo")
        cmd.do("color grey50, SMX2_3_B5zwo")
        cmd.do("color grey50, SMD3_1_B5zwo")
        cmd.do("color grey50, SMD3_2_B5zwo")
        cmd.do("color grey50, SMD3_3_B5zwo")
        cmd.do("color grey50, SMD1_1_B5zwo")
        cmd.do("color grey50, SMD1_2_B5zwo")
        cmd.do("color grey50, SMD1_3_B5zwo")
        cmd.do("color grey50, SMD2_1_B5zwo")
        cmd.do("color grey50, SMD2_2_B5zwo")
        cmd.do("color grey50, SMD2_3_B5zwo")
        cmd.do("color grey50, PRP9_B5zwo")
        cmd.do("color grey50, PRP21_B5zwo")
        cmd.do("color grey50, SNU23_B5zwo")
        cmd.do("color grey50, PRP38_B5zwo")
        cmd.do("color grey50, SPP381_B5zwo")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_Ba5gm6")
        cmd.do("color grey60, BRR2_Ba5gm6")
        cmd.do("color dirtyviolet, BUD31_Ba5gm6")
        cmd.do("color raspberry, CEF1_Ba5gm6")
        cmd.do("color orange, CWC15_Ba5gm6")
        cmd.do("color ruby, CWC2_hRBM22_Ba5gm6")
        cmd.do("color violetpurple, CWC21_Ba5gm6")
        cmd.do("color bluewhite, CWC22_Ba5gm6")
        cmd.do("color lightpink, PRP45_Ba5gm6")
        cmd.do("color dirtyviolet, CDC40_Ba5gm6")
        cmd.do("color grey70, PRP19_Ba5gm6")
        cmd.do("color lightblue, PRP46_Ba5gm6")
        cmd.do("color chocolate, SLT11/ECM2_Ba5gm6")
        cmd.do("color grey70, SNT309_Ba5gm6")
        cmd.do("color slate, SNU114_Ba5gm6")
        cmd.do("color brightorange, SYF2_Ba5gm6")
        cmd.do("color brightorange, SYF1_Ba5gm6")
        cmd.do("color forest, U2_Ba5gm6")
        cmd.do("color density, U5_Ba5gm6")
        cmd.do("color firebrick, U6_Ba5gm6")
        cmd.do("color black, Intron_Ba5gm6")
        cmd.do("color yellow, Exon_Ba5gm6")
        cmd.do("color grey60, BUD13_Ba5gm6")
        cmd.do("color rasberry, CLF2_Ba5gm6")
        cmd.do("color palegreen, Cus1_Ba5gm6")
        cmd.do("color grey60, CWC24_Ba5gm6")
        cmd.do("color grey60, CWC27_Ba5gm6")
        cmd.do("color smudge, HSH155_Ba5gm6")
        cmd.do("color sand, HSH49_Ba5gm6")
        cmd.do("color grey60, PML1_Ba5gm6")
        cmd.do("color palegreen, PRP11_Ba5gm6")
        cmd.do("color palegreen, PRP2_Ba5gm6")
        cmd.do("color palegreen, RDS3_Ba5gm6")
        cmd.do("color smudge, RSE1_Ba5gm6")
        cmd.do("color grey60, SNU17_Ba5gm6")
        cmd.do("color palegreen, Ysf3_Ba5gm6")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_C5lj3")
        cmd.do("color dirtyviolet, BUD31_C5lj3")
        cmd.do("color raspberry, CEF1_C5lj3")
        cmd.do("color raspberry, CLF1_C5lj3")
        cmd.do("color orange, CWC15_C5lj3")
        cmd.do("color lightteal, CWC16/YJU2_C5lj3")
        cmd.do("color ruby, CWC2_hRBM22_C5lj3")
        cmd.do("color violetpurple, CWC21_C5lj3")
        cmd.do("color bluewhite, CWC22_C5lj3")
        cmd.do("color deepteal, CWC25_C5lj3")
        cmd.do("color dirtyviolet, ISY1_C5lj3")
        cmd.do("color palegreen, LEA1_C5lj3")
        cmd.do("color palegreen, Msl1_C5lj3")
        cmd.do("color lightpink, PRP45_C5lj3")
        cmd.do("color lightblue, PRP46_C5lj3")
        cmd.do("color chocolate, SLT11/ECM2_C5lj3")
        cmd.do("color slate, SNU114_C5lj3")
        cmd.do("color brightorange, SYF1_C5lj3")
        cmd.do("color forest, U2_C5lj3")
        cmd.do("color density, U5_C5lj3")
        cmd.do("color firebrick, U6_C5lj3")
        cmd.do("color black, Intron_C5lj3")
        cmd.do("color yellow, Exon_C5lj3")
        cmd.do("color grey50, SMB1_1_C5lj3")
        cmd.do("color grey50, SMB1_2_C5lj3")
        cmd.do("color grey50, SME1_1_C5lj3")
        cmd.do("color grey50, SME1_2_C5lj3")
        cmd.do("color grey50, SMX3_1_C5lj3")
        cmd.do("color grey50, SMX3_2_C5lj3")
        cmd.do("color grey50, SMX2_1_C5lj3")
        cmd.do("color grey50, SMX2_2_C5lj3")
        cmd.do("color grey50, SMD3_1_C5lj3")
        cmd.do("color grey50, SMD3_2_C5lj3")
        cmd.do("color grey50, SMD1_1_C5lj3")
        cmd.do("color grey50, SMD1_2_C5lj3")
        cmd.do("color grey50, SMD2_1_C5lj3")
        cmd.do("color grey50, SMD2_2_C5lj3")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_Cs5mps")
        cmd.do("color dirtyviolet, BUD31_Cs5mps")
        cmd.do("color raspberry, CEF1_Cs5mps")
        cmd.do("color raspberry, CLF1_Cs5mps")
        cmd.do("color orange, CWC15_Cs5mps")
        cmd.do("color ruby, CWC2_hRBM22_Cs5mps")
        cmd.do("color violetpurple, CWC21_Cs5mps")
        cmd.do("color bluewhite, CWC22_Cs5mps")
        cmd.do("color lightpink, PRP45_Cs5mps")
        cmd.do("color dirtyviolet, CDC40_Cs5mps")
        cmd.do("color lightblue, PRP46_Cs5mps")
        cmd.do("color chocolate, SLT11/ECM2_Cs5mps")
        cmd.do("color slate, SNU114_Cs5mps")
        cmd.do("color brightorange, SYF2_Cs5mps")
        cmd.do("color brightorange, SYF1_Cs5mps")
        cmd.do("color forest, U2_Cs5mps")
        cmd.do("color density, U5_Cs5mps")
        cmd.do("color firebrick, U6_Cs5mps")
        cmd.do("color grey50, 5EXON_Cs5mps")
        cmd.do("color black, Intron_Cs5mps")
        cmd.do("color yellow, Exon_Cs5mps")
        cmd.do("color grey50, SMB1_Cs5mps")
        cmd.do("color grey50, SME1_Cs5mps")
        cmd.do("color grey50, SMX3_Cs5mps")
        cmd.do("color grey50, SMX2_Cs5mps")
        cmd.do("color grey50, SMD3_Cs5mps")
        cmd.do("color grey50, SMD1_Cs5mps")
        cmd.do("color grey50, SMD2_Cs5mps")
        cmd.do("color grey50, PRP18_Cs5mps")
        cmd.do("color grey50, SLU7_Cs5mps")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_P6exn")
        cmd.do("color dirtyviolet, BUD31_P6exn")
        cmd.do("color raspberry, CEF1_P6exn")
        cmd.do("color raspberry, CLF1_P6exn")
        cmd.do("color orange, CWC15_P6exn")
        cmd.do("color lightteal, CWC16/YJU2_P6exn")
        cmd.do("color ruby, CWC2_hRBM22_P6exn")
        cmd.do("color violetpurple, CWC21_P6exn")
        cmd.do("color bluewhite, CWC22_P6exn")
        cmd.do("color palegreen, LEA1_P6exn")
        cmd.do("color palegreen, Msl1_P6exn")
        cmd.do("color lightpink, PRP45_P6exn")
        cmd.do("color dirtyviolet, CDC40_P6exn")
        cmd.do("color grey70, PRP19_1_P6exn")
        cmd.do("color grey70, PRP19_2_P6exn")
        cmd.do("color grey70, PRP19_3_P6exn")
        cmd.do("color grey70, PRP19_4_P6exn")
        cmd.do("color lightblue, PRP46_P6exn")
        cmd.do("color chocolate, SLT11/ECM2_P6exn")
        cmd.do("color slate, SNU114_P6exn")
        cmd.do("color brightorange, SYF1_P6exn")
        cmd.do("color forest, U2_P6exn")
        cmd.do("color density, U5_P6exn")
        cmd.do("color firebrick, U6_P6exn")
        cmd.do("color black, Intron_P6exn")
        cmd.do("color yellow, Exon_P6exn")
        cmd.do("color grey50, SMB1_1_P6exn")
        cmd.do("color grey50, SMB1_2_P6exn")
        cmd.do("color grey50, SME1_1_P6exn")
        cmd.do("color grey50, SME1_2_P6exn")
        cmd.do("color grey50, SMX3_1_P6exn")
        cmd.do("color grey50, SMX3_2_P6exn")
        cmd.do("color grey50, SMX2_1_P6exn")
        cmd.do("color grey50, SMX2_2_P6exn")
        cmd.do("color grey50, SMD3_1_P6exn")
        cmd.do("color grey50, SMD3_2_P6exn")
        cmd.do("color grey50, SMD1_1_P6exn")
        cmd.do("color grey50, SMD1_2_P6exn")
        cmd.do("color grey50, SMD2_1_P6exn")
        cmd.do("color grey50, SMD2_2_P6exn")
        cmd.do("color grey50, PRP22_P6exn")
        cmd.do("color grey50, PRP18_P6exn")
        cmd.do("color grey50, SLU7_P6exn")
        cmd.do("color grey50, unassigned_P6exn")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_P5ylz")
        cmd.do("color dirtyviolet, BUD31_P5ylz")
        cmd.do("color raspberry, CEF1_P5ylz")
        cmd.do("color raspberry, CLF1_P5ylz")
        cmd.do("color orange, CWC15_P5ylz")
        cmd.do("color ruby, CWC2_hRBM22_P5ylz")
        cmd.do("color violetpurple, CWC21_P5ylz")
        cmd.do("color bluewhite, CWC22_P5ylz")
        cmd.do("color palegreen, LEA1_P5ylz")
        cmd.do("color palegreen, Msl1_P5ylz")
        cmd.do("color lightpink, PRP45_P5ylz")
        cmd.do("color dirtyviolet, CDC40_P5ylz")
        cmd.do("color grey70, PRP19_1_P5ylz")
        cmd.do("color grey70, PRP19_2_P5ylz")
        cmd.do("color grey70, PRP19_3_P5ylz")
        cmd.do("color grey70, PRP19_4_P5ylz")
        cmd.do("color lightblue, PRP46_P5ylz")
        cmd.do("color chocolate, SLT11/ECM2_P5ylz")
        cmd.do("color grey70, SNT309_P5ylz")
        cmd.do("color slate, SNU114_P5ylz")
        cmd.do("color brightorange, SYF2_P5ylz")
        cmd.do("color brightorange, SYF1_P5ylz")
        cmd.do("color forest, U2_P5ylz")
        cmd.do("color density, U5_P5ylz")
        cmd.do("color firebrick, U6_P5ylz")
        cmd.do("color black, Intron_P5ylz")
        cmd.do("color grey50, SMB1_1_P5ylz")
        cmd.do("color grey50, SMB1_2_P5ylz")
        cmd.do("color grey50, SME1_1_P5ylz")
        cmd.do("color grey50, SME1_2_P5ylz")
        cmd.do("color grey50, SMX3_1_P5ylz")
        cmd.do("color grey50, SMX3_2_P5ylz")
        cmd.do("color grey50, SMX2_1_P5ylz")
        cmd.do("color grey50, SMX2_2_P5ylz")
        cmd.do("color grey50, SMD3_1_P5ylz")
        cmd.do("color grey50, SMD3_2_P5ylz")
        cmd.do("color grey50, SMD1_1_P5ylz")
        cmd.do("color grey50, SMD1_2_P5ylz")
        cmd.do("color grey50, SMD2_1_P5ylz")
        cmd.do("color grey50, SMD2_2_P5ylz")
        cmd.do("color grey50, PRP22_P5ylz")
        cmd.do("color grey50, PRP18_P5ylz")
        cmd.do("color grey50, SLU7_P5ylz")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_I5y88")
        cmd.do("color dirtyviolet, BUD31_I5y88")
        cmd.do("color raspberry, CLF1_I5y88")
        cmd.do("color orange, CWC15_I5y88")
        cmd.do("color lightteal, CWC16/YJU2_I5y88")
        cmd.do("color ruby, CWC2_hRBM22_I5y88")
        cmd.do("color deepteal, CWC25_I5y88")
        cmd.do("color black, Intron_2_I5y88")
        cmd.do("color palegreen, LEA1_I5y88")
        cmd.do("color palegreen, Msl1_I5y88")
        cmd.do("color lightpink, PRP45_I5y88")
        cmd.do("color dirtyviolet, CDC40_I5y88")
        cmd.do("color grey70, PRP19_1_I5y88")
        cmd.do("color grey70, PRP19_2_I5y88")
        cmd.do("color grey70, PRP19_3_I5y88")
        cmd.do("color grey70, PRP19_4_I5y88")
        cmd.do("color lightblue, PRP46_I5y88")
        cmd.do("color chocolate, SLT11/ECM2_I5y88")
        cmd.do("color grey70, SNT309_I5y88")
        cmd.do("color slate, SNU114_I5y88")
        cmd.do("color brightorange, SYF2_I5y88")
        cmd.do("color brightorange, SYF1_I5y88")
        cmd.do("color forest, U2_I5y88")
        cmd.do("color density, U5_I5y88")
        cmd.do("color firebrick, U6_I5y88")
        cmd.do("color black, Intron_I5y88")
        cmd.do("color grey50, RNA_I5y88")
        cmd.do("color grey50, cwc23_I5y88")
        cmd.do("color grey50, SPP382_I5y88")
        cmd.do("color grey50, NTR2_I5y88")
        cmd.do("color grey50, PRP43_I5y88")
        cmd.do("color grey50, SMB1_1_I5y88")
        cmd.do("color grey50, SMB1_2_I5y88")
        cmd.do("color grey50, SME1_1_I5y88")
        cmd.do("color grey50, SME1_2_I5y88")
        cmd.do("color grey50, SMX3_1_I5y88")
        cmd.do("color grey50, SMX3_2_I5y88")
        cmd.do("color grey50, SMX2_1_I5y88")
        cmd.do("color grey50, SMX2_2_I5y88")
        cmd.do("color grey50, SMD3_1_I5y88")
        cmd.do("color grey50, SMD3_2_I5y88")
        cmd.do("color grey50, SMD1_1_I5y88")
        cmd.do("color grey50, SMD1_2_I5y88")
        cmd.do("color grey50, SMD2_1_I5y88")
        cmd.do("color grey50, SMD2_2_I5y88")
  if True: # fake if, just a quick hack
        cmd.do("color forest, U2_3jb9")
        cmd.do("color density, U5_3jb9")
        cmd.do("color firebrick, U6_3jb9")
        cmd.do("color black, Intron_1_3jb9")
        cmd.do("color black, Intron_2_3jb9")
        cmd.do("color grey50, Spp42_yPrp8_3jb9")
        cmd.do("color orange, CWF15_yCWC15_3jb9")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hP_6icz")
        cmd.do("color forest, U2_hP_6icz")
        cmd.do("color density, U5_hP_6icz")
        cmd.do("color firebrick, U6_hP_6icz")
        cmd.do("color black, Intron_hP_6icz")
        cmd.do("color grey50, cwc23_hP_6icz")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hBa_6ff7")
        cmd.do("color ruby, CWC2_hRBM22_hBa_6ff7")
        cmd.do("color forest, U2_hBa_6ff7")
        cmd.do("color density, U5_hBa_6ff7")
        cmd.do("color firebrick, U6_hBa_6ff7")
        cmd.do("color black, Intron_hBa_6ff7")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hC_5yzg")
        cmd.do("color ruby, CWC2_hRBM22_hC_5yzg")
        cmd.do("color deepteal, CWC25_hC_5yzg")
        cmd.do("color smudge, PRP16_hDHX38_hC_5yzg")
        cmd.do("color forest, U2_hC_5yzg")
        cmd.do("color density, U5_hC_5yzg")
        cmd.do("color firebrick, U6_hC_5yzg")
        cmd.do("color black, Intron_hC_5yzg")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_hX_5xjc")
        cmd.do("color deepteal, CWC25_hX_5xjc")
        cmd.do("color forest, U2_hX_5xjc")
        cmd.do("color density, U5_hX_5xjc")
        cmd.do("color firebrick, U6_hX_5xjc")
        cmd.do("color black, Intron_hX_5xjc")
        cmd.do("color grey50, PRKRIP1_hX_5xjc")
  if True: # fake if, just a quick hack
        cmd.do("color orange, CWC15_yCs_5mq0")
        cmd.do("color ruby, CWC2_hRBM22_yCs_5mq0")
        cmd.do("color forest, U2_yCs_5mq0")
        cmd.do("color density, U5_yCs_5mq0")
        cmd.do("color firebrick, U6_yCs_5mq0")
        cmd.do("color grey50, 5EXON_yCs_5mq0")
        cmd.do("color black, Intron_yCs_5mq0")
        cmd.do("color yellow, exon-3_yCs_5mq0")
  if True: # fake if, just a quick hack
        cmd.do("color skyblue, PRP8_yC_5lj5")
        cmd.do("color grey60, BRR2_yC_5lj5")
        cmd.do("color dirtyviolet, BUD31_yC_5lj5")
        cmd.do("color raspberry, CEF1_yC_5lj5")
        cmd.do("color raspberry, CLF1_yC_5lj5")
        cmd.do("color orange, CWC15_yC_5lj5")
        cmd.do("color lightteal, CWC16/YJU2_yC_5lj5")
        cmd.do("color ruby, CWC2_hRBM22_yC_5lj5")
        cmd.do("color violetpurple, CWC21_yC_5lj5")
        cmd.do("color bluewhite, CWC22_yC_5lj5")
        cmd.do("color deepteal, CWC25_yC_5lj5")
        cmd.do("color dirtyviolet, ISY1_yC_5lj5")
        cmd.do("color palegreen, Msl1_yC_5lj5")
        cmd.do("color lightpink, PRP45_yC_5lj5")
        cmd.do("color smudge, PRP16_hDHX38_yC_5lj5")
        cmd.do("color grey70, PRP19_1_yC_5lj5")
        cmd.do("color grey70, PRP19_2_yC_5lj5")
        cmd.do("color grey70, PRP19_3_yC_5lj5")
        cmd.do("color grey70, PRP19_4_yC_5lj5")
        cmd.do("color lightblue, PRP46_yC_5lj5")
        cmd.do("color chocolate, SLT11/ECM2_yC_5lj5")
        cmd.do("color grey70, SNT309_yC_5lj5")
        cmd.do("color slate, SNU114_yC_5lj5")
        cmd.do("color brightorange, SYF1_yC_5lj5")
        cmd.do("color forest, U2_yC_5lj5")
        cmd.do("color density, U5_yC_5lj5")
        cmd.do("color firebrick, U6_yC_5lj5")
        cmd.do("color grey50, 5EXON_yC_5lj5")
        cmd.do("color black, Intron_yC_5lj5")
        cmd.do("color grey50, SMB1_1_yC_5lj5")
        cmd.do("color grey50, SMB1_2_yC_5lj5")
        cmd.do("color grey50, SME1_1_yC_5lj5")
        cmd.do("color grey50, SME1_2_yC_5lj5")
        cmd.do("color grey50, SMX3_1_yC_5lj5")
        cmd.do("color grey50, SMX3_2_yC_5lj5")
        cmd.do("color grey50, SMX2_1_yC_5lj5")
        cmd.do("color grey50, SMX2_2_yC_5lj5")
        cmd.do("color grey50, SMD3_1_yC_5lj5")
        cmd.do("color grey50, SMD3_2_yC_5lj5")
        cmd.do("color grey50, SMD1_1_yC_5lj5")
        cmd.do("color grey50, SMD1_2_yC_5lj5")
        cmd.do("color grey50, SMD2_1_yC_5lj5")
        cmd.do("color grey50, SMD2_2_yC_5lj5")
        cmd.do("color grey50, unassigned_yC_5lj5")
  if True: # fake if, just a quick hack
        cmd.do("color palegreen, LEA1_yPre_6g90")
        cmd.do("color palegreen, Msl1_yPre_6g90")
        cmd.do("color palegreen, Ysf3_yPre_6g90")
        cmd.do("color grey50, SMB1_1_yPre_6g90")
        cmd.do("color grey50, SMB1_2_yPre_6g90")
        cmd.do("color grey50, SME1_1_yPre_6g90")
        cmd.do("color grey50, SME1_2_yPre_6g90")
        cmd.do("color grey50, SMX3_1_yPre_6g90")
        cmd.do("color grey50, SMX3_2_yPre_6g90")
        cmd.do("color grey50, SMX2_1_yPre_6g90")
        cmd.do("color grey50, SMX2_2_yPre_6g90")
        cmd.do("color grey50, SMD3_1_yPre_6g90")
        cmd.do("color grey50, SMD3_2_yPre_6g90")
        cmd.do("color grey50, SMD1_1_yPre_6g90")
        cmd.do("color grey50, SMD1_2_yPre_6g90")
        cmd.do("color grey50, SMD2_1_yPre_6g90")
        cmd.do("color grey50, SMD2_2_yPre_6g90")
        cmd.do("color grey50, unassigned_yPre_6g90")
        cmd.do("color grey51, MUD1_yPre_6g90")
        cmd.do("color orange, SNP1_yPre_6g90")
        cmd.do("color grey53, YHC1_yPre_6g90")
        cmd.do("color grey54, PRP39_yPre_6g90")
        cmd.do("color grey55, PRP42_yPre_6g90")
        cmd.do("color grey56, NAM8_yPre_6g90")
        cmd.do("color grey57, SNU56_yPre_6g90")
        cmd.do("color grey58, LUC7_yPre_6g90")
        cmd.do("color grey59, SNU71_yPre_6g90")
        cmd.do("color grey60, HSH155_yPre_6g90")
        cmd.do("color grey61, RSE1_yPre_6g90")
        cmd.do("color grey62, CUS1_yPre_6g90")
        cmd.do("color grey63, HSH49_yPre_6g90")
        cmd.do("color grey64, RDS3_yPre_6g90")
        cmd.do("color grey65, PRP9_yPre_6g90")
        cmd.do("color grey66, PRP11_yPre_6g90")
        cmd.do("color grey67, PRP21_yPre_6g90")
  if True: # fake if, just a quick hack
        cmd.do("color black, Intron_yE_6n7r")
        cmd.do("color grey50, SMB1_yE_6n7r")
        cmd.do("color grey50, SME1_yE_6n7r")
        cmd.do("color grey50, SMX3_yE_6n7r")
        cmd.do("color grey50, SMX2_yE_6n7r")
        cmd.do("color grey50, SMD3_yE_6n7r")
        cmd.do("color grey50, SMD1_yE_6n7r")
        cmd.do("color grey50, SMD2_yE_6n7r")
        cmd.do("color grey51, MUD1_yE_6n7r")
        cmd.do("color orange, SNP1_yE_6n7r")
        cmd.do("color grey53, YHC1_yE_6n7r")
        cmd.do("color grey54, PRP39_yE_6n7r")
        cmd.do("color grey55, PRP42_yE_6n7r")
        cmd.do("color grey56, NAM8_yE_6n7r")
        cmd.do("color grey57, SNU56_yE_6n7r")
        cmd.do("color grey58, LUC7_yE_6n7r")
        cmd.do("color grey59, SNU71_yE_6n7r")
        cmd.do("color green, U1_yE_6n7r")
  if True: # fake if, just a quick hack
        cmd.do("color black, Intron_yE_6n7p")
        cmd.do("color grey50, SMB1_yE_6n7p")
        cmd.do("color grey50, SME1_yE_6n7p")
        cmd.do("color grey50, SMX3_yE_6n7p")
        cmd.do("color grey50, SMX2_yE_6n7p")
        cmd.do("color grey50, SMD3_yE_6n7p")
        cmd.do("color grey50, SMD1_yE_6n7p")
        cmd.do("color grey50, SMD2_yE_6n7p")
        cmd.do("color grey51, MUD1_yE_6n7p")
        cmd.do("color orange, SNP1_yE_6n7p")
        cmd.do("color grey53, YHC1_yE_6n7p")
        cmd.do("color grey54, PRP39_yE_6n7p")
        cmd.do("color grey55, PRP42_yE_6n7p")
        cmd.do("color grey56, NAM8_yE_6n7p")
        cmd.do("color grey57, SNU56_yE_6n7p")
        cmd.do("color grey58, LUC7_yE_6n7p")
        cmd.do("color grey59, SNU71_yE_6n7p")
        cmd.do("color green, U1_yE_6n7p")
        cmd.do("color grey69, PRP40_yE_6n7p")
        cmd.do("color grey70, STO1_yE_6n7p")
        cmd.do("color grey71, CBC2_yE_6n7p")
